@@ -7,21 +7,19 @@ import random
 import string
 import typing as ty
 import warnings
-
-import contextlib
+import time
+import numexpr
+import numpy as np
+import pandas as pd
+import strax
 import sys
 if any('jupyter' in arg for arg in sys.argv):
     # In some cases we are not using any notebooks,
     # Taken from 44952863 on stack overflow thanks!
-    from tqdm import tqdm_notebook as tqdm
+    from tqdm.notebook import tqdm
 else:
     from tqdm import tqdm
 
-import numexpr
-import numpy as np
-import pandas as pd
-
-import strax
 export, __all__ = strax.exporter()
 __all__ += ['RUN_DEFAULTS_KEY']
 
@@ -69,6 +67,9 @@ RUN_DEFAULTS_KEY = 'strax_defaults'
     strax.Option(name='timeout', default=24 * 3600,
                  help="Terminate processing if any one mailbox receives "
                       "no result for more than this many seconds"),
+    strax.Option(name='saver_timeout', default=900,
+                 help="Max time [s] a saver can take to store a result. Set "
+                      "high for slow compression algorithms."),
     strax.Option(name='use_per_run_defaults', default=False,
                  help='Scan the run db for per-run defaults. '
                       'This is an experimental strax feature that will '
@@ -359,7 +360,7 @@ class Context:
 
         for x in dir(module):
             x = getattr(module, x)
-            if type(x) != type(type):
+            if not isinstance(x, type(type)):
                 continue
             if issubclass(x, strax.Plugin):
                 self.register(x)
@@ -425,7 +426,6 @@ class Context:
                     assert parent_name in p.config, mes
                     p.config[parent_name] = option_value
 
-
     def _get_plugins(self,
                      targets: ty.Tuple[str],
                      run_id: str) -> ty.Dict[str, strax.Plugin]:
@@ -525,6 +525,19 @@ class Context:
 
         return plugins
 
+    @staticmethod
+    def _get_end_targets(plugins: dict) -> ty.Tuple[str]:
+        """
+        Get the datatype that is provided by a plugin but not depended
+            on by any other plugin
+        """
+        provides = [prov for p in plugins.values()
+                    for prov in strax.to_str_tuple(p.provides)]
+        depends_on = [dep for p in plugins.values()
+                      for dep in strax.to_str_tuple(p.depends_on)]
+        uniques = list(set(provides) ^ set(depends_on))
+        return strax.to_str_tuple(uniques)
+
     @property
     def _find_options(self):
 
@@ -570,7 +583,7 @@ class Context:
 
     def get_components(self, run_id: str,
                        targets=tuple(), save=tuple(),
-                       time_range=None, chunk_number=None
+                       time_range=None, chunk_number=None,
                        ) -> strax.ProcessorComponents:
         """Return components for setting up a processor
         {get_docs}
@@ -579,15 +592,11 @@ class Context:
         save = strax.to_str_tuple(save)
         targets = strax.to_str_tuple(targets)
 
-        # Although targets is a tuple, we only support one target at the moment
-        # we could just make it a string!
-        assert len(targets) == 1, f"Found {len(targets)} instead of 1 target"
-        if len(targets[0]) == 1:
-            raise ValueError(
-                f"Plugin names must be more than one letter, not {targets[0]}")
+        for t in targets:
+            if len(t) == 1:
+                raise ValueError(f"Plugin names must be more than one letter, not {t}")
 
         plugins = self._get_plugins(targets, run_id)
-        target = targets[0]  # See above, already restricted to one target
 
         # Get savers/loaders, and meanwhile filter out plugins that do not
         # have to do computation. (their instances will stick around
@@ -747,7 +756,9 @@ class Context:
                             key,
                             metadata=p.metadata(
                                 run_id,
-                                d_to_save)))
+                                d_to_save),
+                            saver_timeout=self.context_config['saver_timeout']
+                        ))
                     except strax.DataNotAvailable:
                         # This frontend cannot save. Too bad.
                         pass
@@ -759,7 +770,14 @@ class Context:
         intersec = list(plugins.keys() & loaders.keys())
         if len(intersec):
             raise RuntimeError(f"{intersec} both computed and loaded?!")
-
+        if len(targets) > 1:
+            final_plugin = [t for t in targets if t in self._get_end_targets(plugins)][:1]
+            self.log.warning(
+                f'Multiple targets detected! This is only suitable for mass '
+                f'producing dataypes since only {final_plugin} will be '
+                f'subscribed in the mailbox system!')
+        else:
+            final_plugin = targets
         # For the plugins which will run computations,
         # check all required options are available or set defaults.
         # Also run any user-defined setup
@@ -770,25 +788,14 @@ class Context:
             plugins=plugins,
             loaders=loaders,
             savers=dict(savers),
-            targets=targets)
-
-    def estimate_run_start(self, run_id, targets=None):
-        """Return run start time in ns since epoch.
-
-        This fetches from run metadata, and if this fails, it
-        estimates it using data metadata from targets.
-        """
-        warnings.warn('This function is outdated and will be removed in '
-                      'one of the feature strax releases. Please use '
-                      '"estimate_run_start_and_end" instead.', UserWarning)
-        start, _ = self.estimate_run_start_and_end(run_id, targets=targets)
-        return start
+            targets=strax.to_str_tuple(final_plugin))
 
     def estimate_run_start_and_end(self, run_id, targets=None):
         """Return run start and end time in ns since epoch.
 
-        This fetches from run metadata, and if this fails, it
-        estimates it using data metadata from targets.
+        This fetches from run metadata, and if this fails, it estimates
+            it using data metadata from the targets or the underlying
+            data-types (if it is stored).
         """
         try:
             res = []
@@ -800,12 +807,19 @@ class Context:
                 t = int(t.timestamp()) * int(1e9)
                 res.append(t)
             return res
-        except (strax.RunMetadataNotAvailable, KeyError):
+        except (strax.RunMetadataNotAvailable, KeyError) as e:
+            self.log.debug(f'Could not infer start/stop due to type {type(e)} {e}')
             pass
         # Get an approx start from the data itself,
         # then floor it to seconds for consistency
         if targets:
-            for t in strax.to_str_tuple(targets):
+            self.log.debug('Infer start/stop from targets')
+            for t in self._get_plugins(strax.to_str_tuple(targets),
+                                       run_id,
+                                       ).keys():
+                if not self.is_stored(run_id, t):
+                    continue
+                self.log.debug(f'Try inferring start/stop from {t}')
                 try:
                     t0 = self.get_meta(run_id, t)['chunks'][0]['start']
                     t0 = (int(t0) // int(1e9)) * int(1e9)
@@ -815,10 +829,9 @@ class Context:
                     return t0, t1
                 except strax.DataNotAvailable:
                     pass
-        warnings.warn(
+        self.log.warning(
             "Could not estimate run start and end time from "
-            "run metadata: assuming it is 0 and inf",
-            UserWarning)
+            "run metadata: assuming it is 0 and inf")
         return 0, float('inf')
 
     def to_absolute_time_range(self, run_id, targets=None, time_range=None,
@@ -868,8 +881,9 @@ class Context:
                  time_selection='fully_contained',
                  selection_str=None,
                  keep_columns=None,
-                 _chunk_number=None,
+                 allow_multiple=False,
                  progress_bar=True,
+                 _chunk_number=None,
                  **kwargs) -> ty.Iterator[strax.Chunk]:
         """Compute target for run_id and iterate over results.
 
@@ -902,8 +916,16 @@ class Context:
                          dict(depends_on=tuple(targets)))
                 self.register(p)
                 targets = (temp_name,)
-            else:
+            elif not allow_multiple:
                 raise RuntimeError("Cannot automerge different data kinds!")
+            elif (self.context_config['timeout'] > 7200 or (
+                    self.context_config['allow_lazy'] and
+                    not self.context_config['allow_multiprocess'])):
+                # For allow_multiple we don't want allow this when in lazy mode
+                # with long timeouts (lazy-mode is disabled if multiprocessing
+                # so if that is activated, we can also continue)
+                raise RuntimeError(f'Cannot allow_multiple in lazy mode or '
+                                   f'with long timeouts.')
 
         components = self.get_components(run_id,
                                          targets=targets,
@@ -926,46 +948,13 @@ class Context:
                 allow_lazy=self.context_config['allow_lazy'],
                 max_messages=self.context_config['max_messages'],
                 timeout=self.context_config['timeout']).iter()
-        
-        # Defining time ranges for the progress bar:
-        if time_range:
-            # user specified a time selection
-            start_time, end_time = time_range
-        else:
-            # If no selection is specified we have to get the last end_time:
-            start_time = 0
-            end_time = float('inf')
-
-        if progress_bar:
-            for t in strax.to_str_tuple(targets):
-                try:
-                    # Sometimes some metadata might be missing e.g. during tests.
-                    chunks = self.get_meta(run_id, t)['chunks']
-                    start_time = max(start_time, chunks[0]['start'])
-                    end_time = min(end_time, chunks[-1]['end'])
-                except (strax.DataNotAvailable, KeyError, IndexError):
-                    # IndexError caused for only empty chunks when data not
-                    # available.
-                    # Maybe at least one target had some metadata.
-                    start_time = max(start_time, 0)
-                    end_time = min(end_time, float('inf'))
-
-            # Define nice progressbar format:
-            bar_format = "{desc}: |{bar}| {percentage:.2f} % [{elapsed}<{remaining}],"\
-                         " {postfix[0]} {postfix[1][spc]:.2f} s/chunk,"\
-                         " #chunks processed: {postfix[1][n]}"
-            sec_per_chunk = np.nan  # Have not computed any chunk yet.
-            post_fix = ['Rate last Chunk:', {'spc': sec_per_chunk, 'n': 0}]
 
         try:
-            with contextlib.ExitStack() as stack:
-                if progress_bar and (start_time != 0 and end_time != float('inf')):
-                    # Get initial time
-                    pbar = stack.enter_context(tqdm(total=1, postfix=post_fix, bar_format=bar_format))
-                    last_time = pbar.last_print_t
-                else:
-                    progress_bar = False
-                
+            _p, t_start, t_end = self._make_progress_bar(run_id,
+                                                         targets=targets,
+                                                         progress_bar=progress_bar)
+            with _p as pbar:
+                pbar.last_print_t = time.time()
                 for n_chunks, result in enumerate(strax.continuity_check(generator), 1):
                     seen_a_chunk = True
                     if not isinstance(result, strax.Chunk):
@@ -977,27 +966,11 @@ class Context:
                         keep_columns=keep_columns,
                         time_range=time_range,
                         time_selection=time_selection)
-                    
-                    if progress_bar:
-                        # Update progressbar:
-                        if end_time - start_time > 0:
-                            pbar.n = (result.end - start_time) / (end_time - start_time)
-                        else:
-                            # Strange, start and endtime are the same, probably we 
-                            # don't have data yet e.g. allow_incomplete == True.
-                            pbar.n = 0
-                        pbar.update(0)
-                        # Now get last time printed and refresh seconds_per_chunk:
-                        # This is a small work around since we do not know the
-                        # pacemaker here and therefore we do not know the number of
-                        # chunks.
-                        sec_per_chunk = pbar.last_print_t - last_time
-                        pbar.postfix[1]['spc'] = sec_per_chunk
-                        pbar.postfix[1]['n'] = n_chunks
-                        pbar.refresh()
-                        last_time = pbar.last_print_t
+                    self._update_progress_bar(
+                        pbar, t_start, t_end, n_chunks, result.end)
 
                     yield result
+            _p.close()
 
         except GeneratorExit:
             generator.throw(OutsideException(
@@ -1014,6 +987,55 @@ class Context:
             raise ValueError(f"Invalid time range: {time_range}, "
                              "returned no chunks!")
 
+    def _make_progress_bar(self, run_id, targets, progress_bar=True):
+        """
+        Make a progress bar for get_iter
+        :param run_id, targets: run_id and targets
+        :param progress_bar: Bool whether or not to display the progress bar
+        :return: progress bar, t_start (run) and t_end (run)
+        """
+        try:
+            t_start, t_end, = self.estimate_run_start_and_end(run_id, targets)
+        except (AttributeError, KeyError, IndexError):
+            # During testing some thing remain a secret
+            t_start, t_end, = 0, float('inf')
+        if t_end == float('inf'):
+            progress_bar = False
+
+        # Define nice progressbar format:
+        bar_format = ("{desc}: "  # The loading plugin x
+                      "|{bar}| "  # Bar that is being filled
+                      "{percentage:.2f} % "  # Percentage
+                      "[{elapsed}<{remaining}]"  # Time estimate
+                      "{postfix}"  # Extra info
+                      )
+        description = f'Loading {"plugins" if targets[0].startswith("_temp") else targets}'
+        pbar = tqdm(total=1,
+                    desc=description,
+                    bar_format=bar_format,
+                    leave=True,
+                    disable=not progress_bar)
+        return pbar, t_start, t_end
+
+    @staticmethod
+    def _update_progress_bar(pbar, t_start, t_end, n_chunks, chunk_end):
+        """Do some tqdm voodoo to get the progress bar for st.get_iter"""
+        if t_end - t_start > 0:
+            fraction_done = (chunk_end - t_start) / (t_end - t_start)
+            if fraction_done > .99:
+                # Patch to 1 to not have a red pbar when very close to 100%
+                fraction_done = 1
+            pbar.n = np.clip(fraction_done, 0, 1)
+        else:
+            # Strange, start and endtime are the same, probably we don't
+            # have data yet e.g. allow_incomplete == True.
+            pbar.n = 0
+        # Let's add the postfix which is the info behind the tqdm marker
+        seconds_per_chunk = time.time() - pbar.last_print_t
+        postfix = f'Last chunk: {seconds_per_chunk:.2f} s. #chunks {n_chunks}'
+        pbar.set_postfix_str(postfix)
+        pbar.update(0)
+
     def apply_selection(self, x,
                         selection_str=None,
                         keep_columns=None,
@@ -1108,6 +1130,10 @@ class Context:
         {get_docs}
         """
         run_ids = strax.to_str_tuple(run_id)
+
+        if kwargs.get('allow_multiple', False):
+            raise RuntimeError('Cannot allow_multiple with get_array/get_df')
+
         if len(run_ids) > 1:
             results = strax.multi_run(
                 self.get_array, run_ids, targets=targets,
@@ -1168,6 +1194,9 @@ class Context:
                 n_chunks: number of chunks in run
                 n_rows: number of data entries in run
         """
+        if kwargs.get('allow_multiple', False):
+            raise RuntimeError('Cannot allow_multiple with accumulate')
+
         n_chunks = 0
         seen_data = False
         result = {'n_rows': 0}
@@ -1178,7 +1207,8 @@ class Context:
                 return arr
             function_takes_fields = False
 
-        for chunk in self.get_iter(run_id, targets, **kwargs):
+        for chunk in self.get_iter(run_id, targets,
+                                   **kwargs):
             data = chunk.data
             data = self._apply_function(data, targets)
 
@@ -1472,6 +1502,18 @@ class Context:
             raise ValueError('This cannot happen, we just checked that this '
                              'run should be stored?!?')
 
+    def provided_dtypes(self, runid='0'):
+        """
+        Summarize useful dtype information provided by this context
+        :return: dictionary of provided dtypes with their corresponding lineage hash, save_when, version
+        """
+        hashes = set([(d, self.key_for(runid, d).lineage_hash, p.save_when, p.__version__)
+                    for p in self._plugin_class_registry.values()
+                    for d in p.provides])
+
+        return {dtype: dict(hash=h, save_when=save_when.name, version=version)
+                for dtype, h, save_when, version in hashes}
+
     @classmethod
     def add_method(cls, f):
         """Add f as a new Context method"""
@@ -1480,6 +1522,8 @@ class Context:
 
 select_docs = """
 :param selection_str: Query string or sequence of strings to apply.
+:param keep_columns: Array field/dataframe column names to keep. 
+    Useful to reduce amount of data in memory.
 :param time_range: (start, stop) range to load, in ns since the epoch
 :param seconds_range: (start, stop) range of seconds since
 the start of the run to load.
@@ -1500,6 +1544,11 @@ get_docs = """
     Many plugins save automatically anyway.
 :param max_workers: Number of worker threads/processes to spawn.
     In practice more CPUs may be used due to strax's multithreading.
+:param allow_multiple: Allow multiple targets to be computed
+    simultaneously without merging the results of the target. This can
+    be used when mass producing plugins that are not of the same
+    datakind. Don't try to use this in get_array or get_df because the
+    data is not returned.
 """ + select_docs
 
 
