@@ -62,6 +62,7 @@ def mailbox_tester(messages,
 def test_highlevel():
     """Test highlevel mailbox API"""
     for lazy in [False, True]:
+        n_threads_start = len(threading.enumerate())
         print(f"Lazy mode: {lazy}")
 
         mb = strax.Mailbox(lazy=lazy)
@@ -78,7 +79,10 @@ def test_highlevel():
         assert hasattr(test_reader, 'got')
         assert test_reader.got == list(range(10))
         mb.cleanup()
-        assert len(threading.enumerate()) == 1, "Not all threads died"
+        threads = [f'{t.name} is dead: {True^t.is_alive()}'
+                   for t in threading.enumerate()]
+        assert len(threads) == n_threads_start, (
+            f"Not all threads died. \n Threads running are:{threads}")
 
 
 def test_result_timeout():
